@@ -90,7 +90,7 @@ class FunctionsGLWindows : public FunctionsGL
 DisplayWGL::DisplayWGL(const egl::DisplayState &state)
     : DisplayGL(state),
       mRenderer(nullptr),
-      mCurrentData(),
+      mCurrentNativeContexts(),
       mOpenGLModule(nullptr),
       mFunctionsWGL(nullptr),
       mHasWGLCreateContextRobustness(false),
@@ -136,7 +136,7 @@ egl::Error DisplayWGL::initializeImpl(egl::Display *display)
     mFunctionsWGL->initialize(mOpenGLModule, nullptr);
 
     // WGL can't grab extensions until it creates a context because it needs to load the driver's
-    // DLLs first. Create a dummy context to load the driver and determine which GL versions are
+    // DLLs first. Create a stub context to load the driver and determine which GL versions are
     // available.
 
     // Work around compile error from not defining "UNICODE" while Chromium does
@@ -165,60 +165,61 @@ egl::Error DisplayWGL::initializeImpl(egl::Display *display)
                << "\":" << gl::FmtErr(HRESULT_CODE(GetLastError()));
     }
 
-    HWND dummyWindow =
-        CreateWindowExA(0, reinterpret_cast<const char *>(mWindowClass), "ANGLE Dummy Window",
+    HWND placeholderWindow =
+        CreateWindowExA(0, reinterpret_cast<const char *>(mWindowClass), "ANGLE Placeholder Window",
                         WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
                         CW_USEDEFAULT, nullptr, nullptr, nullptr, nullptr);
-    if (!dummyWindow)
+    if (!placeholderWindow)
     {
-        return egl::EglNotInitialized() << "Failed to create dummy OpenGL window.";
+        return egl::EglNotInitialized() << "Failed to create placeholder OpenGL window.";
     }
 
-    HDC dummyDeviceContext = GetDC(dummyWindow);
-    if (!dummyDeviceContext)
+    HDC placeholderDeviceContext = GetDC(placeholderWindow);
+    if (!placeholderDeviceContext)
     {
         return egl::EglNotInitialized()
-               << "Failed to get the device context of the dummy OpenGL window.";
+               << "Failed to get the device context of the placeholder OpenGL window.";
     }
 
     const PIXELFORMATDESCRIPTOR pixelFormatDescriptor = wgl::GetDefaultPixelFormatDescriptor();
 
-    int dummyPixelFormat = ChoosePixelFormat(dummyDeviceContext, &pixelFormatDescriptor);
-    if (dummyPixelFormat == 0)
+    int placeholderPixelFormat =
+        ChoosePixelFormat(placeholderDeviceContext, &pixelFormatDescriptor);
+    if (placeholderPixelFormat == 0)
     {
         return egl::EglNotInitialized()
-               << "Could not find a compatible pixel format for the dummy OpenGL window.";
+               << "Could not find a compatible pixel format for the placeholder OpenGL window.";
     }
 
-    if (!SetPixelFormat(dummyDeviceContext, dummyPixelFormat, &pixelFormatDescriptor))
+    if (!SetPixelFormat(placeholderDeviceContext, placeholderPixelFormat, &pixelFormatDescriptor))
     {
         return egl::EglNotInitialized()
                << "Failed to set the pixel format on the intermediate OpenGL window.";
     }
 
-    HGLRC dummyWGLContext = mFunctionsWGL->createContext(dummyDeviceContext);
-    if (!dummyDeviceContext)
+    HGLRC placeholderWGLContext = mFunctionsWGL->createContext(placeholderDeviceContext);
+    if (!placeholderDeviceContext)
     {
         return egl::EglNotInitialized()
-               << "Failed to create a WGL context for the dummy OpenGL window.";
+               << "Failed to create a WGL context for the placeholder OpenGL window.";
     }
 
-    if (!mFunctionsWGL->makeCurrent(dummyDeviceContext, dummyWGLContext))
+    if (!mFunctionsWGL->makeCurrent(placeholderDeviceContext, placeholderWGLContext))
     {
-        return egl::EglNotInitialized() << "Failed to make the dummy WGL context current.";
+        return egl::EglNotInitialized() << "Failed to make the placeholder WGL context current.";
     }
 
     // Reinitialize the wgl functions to grab the extensions
-    mFunctionsWGL->initialize(mOpenGLModule, dummyDeviceContext);
+    mFunctionsWGL->initialize(mOpenGLModule, placeholderDeviceContext);
 
     mHasWGLCreateContextRobustness =
         mFunctionsWGL->hasExtension("WGL_ARB_create_context_robustness");
 
-    // Destroy the dummy window and context
-    mFunctionsWGL->makeCurrent(dummyDeviceContext, nullptr);
-    mFunctionsWGL->deleteContext(dummyWGLContext);
-    ReleaseDC(dummyWindow, dummyDeviceContext);
-    DestroyWindow(dummyWindow);
+    // Destroy the placeholder window and context
+    mFunctionsWGL->makeCurrent(placeholderDeviceContext, nullptr);
+    mFunctionsWGL->deleteContext(placeholderWGLContext);
+    ReleaseDC(placeholderWindow, placeholderDeviceContext);
+    DestroyWindow(placeholderWindow);
 
     const egl::AttributeMap &displayAttributes = display->getAttributeMap();
     EGLint requestedDisplayType                = static_cast<EGLint>(displayAttributes.get(
@@ -353,7 +354,7 @@ void DisplayWGL::destroy()
             mFunctionsWGL->makeCurrent(mDeviceContext, nullptr);
         }
     }
-    mCurrentData.clear();
+    mCurrentNativeContexts.clear();
 
     SafeDelete(mFunctionsWGL);
 
@@ -471,12 +472,6 @@ rx::ContextImpl *DisplayWGL::createContext(const gl::State &state,
     return new ContextWGL(state, errorSet, mRenderer);
 }
 
-DeviceImpl *DisplayWGL::createDevice()
-{
-    UNREACHABLE();
-    return nullptr;
-}
-
 egl::ConfigSet DisplayWGL::generateConfigs()
 {
     egl::ConfigSet configs;
@@ -587,12 +582,6 @@ egl::Error DisplayWGL::validateClientBuffer(const egl::Config *configuration,
     }
 }
 
-std::string DisplayWGL::getVendorString() const
-{
-    // UNIMPLEMENTED();
-    return "";
-}
-
 egl::Error DisplayWGL::initializeD3DDevice()
 {
     if (mD3D11Device != nullptr)
@@ -645,8 +634,9 @@ void DisplayWGL::generateExtensions(egl::DisplayExtensions *outExtensions) const
     outExtensions->querySurfacePointer            = true;
     outExtensions->keyedMutex                     = true;
 
-    // Contexts are virtualized so textures can be shared globally
-    outExtensions->displayTextureShareGroup = true;
+    // Contexts are virtualized so textures and semaphores can be shared globally
+    outExtensions->displayTextureShareGroup   = true;
+    outExtensions->displaySemaphoreShareGroup = true;
 
     outExtensions->surfacelessContext = true;
 
@@ -677,22 +667,19 @@ egl::Error DisplayWGL::waitNative(const gl::Context *context, EGLint engine)
     return egl::NoError();
 }
 
-egl::Error DisplayWGL::makeCurrent(egl::Surface *drawSurface,
+egl::Error DisplayWGL::makeCurrent(egl::Display *display,
+                                   egl::Surface *drawSurface,
                                    egl::Surface *readSurface,
                                    gl::Context *context)
 {
-    CurrentNativeContext &currentContext = mCurrentData[std::this_thread::get_id()];
+    CurrentNativeContext &currentContext = mCurrentNativeContexts[std::this_thread::get_id()];
 
-    HDC newDC = currentContext.dc;
+    HDC newDC = mDeviceContext;
     if (drawSurface)
     {
         SurfaceWGL *drawSurfaceWGL = GetImplAs<SurfaceWGL>(drawSurface);
         newDC                      = drawSurfaceWGL->getDC();
     }
-    else
-    {
-        newDC = mDeviceContext;
-    }
 
     HGLRC newContext = currentContext.glrc;
     if (context)
@@ -718,7 +705,7 @@ egl::Error DisplayWGL::makeCurrent(egl::Surface *drawSurface,
         currentContext.glrc = newContext;
     }
 
-    return DisplayGL::makeCurrent(drawSurface, readSurface, context);
+    return DisplayGL::makeCurrent(display, drawSurface, readSurface, context);
 }
 
 egl::Error DisplayWGL::registerD3DDevice(IUnknown *device, HANDLE *outHandle)
@@ -914,7 +901,7 @@ egl::Error DisplayWGL::createRenderer(std::shared_ptr<RendererWGL> *outRenderer)
     {
         return egl::EglNotInitialized() << "Failed to make the intermediate WGL context current.";
     }
-    CurrentNativeContext &currentContext = mCurrentData[std::this_thread::get_id()];
+    CurrentNativeContext &currentContext = mCurrentNativeContexts[std::this_thread::get_id()];
     currentContext.dc                    = mDeviceContext;
     currentContext.glrc                  = context;
 
@@ -1062,4 +1049,9 @@ void DisplayWGL::populateFeatureList(angle::FeatureList *features)
     mRenderer->getFeatures().populateFeatureList(features);
 }
 
+RendererGL *DisplayWGL::getRenderer() const
+{
+    return reinterpret_cast<RendererGL *>(mRenderer.get());
+}
+
 }  // namespace rx
