@@ -30,7 +30,10 @@
 #include "URLParser.h"
 #include <stdio.h>
 #include <unicode/uidna.h>
+#include <wtf/FileSystem.h>
 #include <wtf/HashMap.h>
+#include <wtf/HashSet.h>
+#include <wtf/Lock.h>
 #include <wtf/NeverDestroyed.h>
 #include <wtf/PrintStream.h>
 #include <wtf/StdLibExtras.h>
@@ -38,6 +41,7 @@
 #include <wtf/text/CString.h>
 #include <wtf/text/StringBuilder.h>
 #include <wtf/text/StringHash.h>
+#include <wtf/text/StringToIntegerConversion.h>
 #include <wtf/text/TextStream.h>
 
 namespace WTF {
@@ -69,13 +73,20 @@ static bool shouldTrimFromURL(UChar character)
     return character <= ' ';
 }
 
-URL URL::isolatedCopy() const
+URL URL::isolatedCopy() const &
 {
     URL result = *this;
     result.m_string = result.m_string.isolatedCopy();
     return result;
 }
 
+URL URL::isolatedCopy() &&
+{
+    URL result = *this;
+    result.m_string = WTFMove(result.m_string).isolatedCopy();
+    return result;
+}
+
 StringView URL::lastPathComponent() const
 {
     if (!hasPath())
@@ -93,6 +104,27 @@ StringView URL::lastPathComponent() const
     return StringView(m_string).substring(start, end - start + 1);
 }
 
+bool URL::hasSpecialScheme() const
+{
+    // https://url.spec.whatwg.org/#special-scheme
+    return protocolIs("ftp")
+        || protocolIs("file")
+        || protocolIs("http")
+        || protocolIs("https")
+        || protocolIs("ws")
+        || protocolIs("wss");
+}
+
+unsigned URL::pathStart() const
+{
+    unsigned start = m_hostEnd + m_portLength;
+    if (start == m_schemeEnd + 1U
+        && start + 1 < m_string.length()
+        && m_string[start] == '/' && m_string[start + 1] == '.')
+        start += 2;
+    return start;
+}
+
 StringView URL::protocol() const
 {
     if (!m_isValid)
@@ -110,9 +142,9 @@ StringView URL::host() const
     return StringView(m_string).substring(start, m_hostEnd - start);
 }
 
-Optional<uint16_t> URL::port() const
+std::optional<uint16_t> URL::port() const
 {
-    return m_portLength ? parseUInt16(StringView(m_string).substring(m_hostEnd + 1, m_portLength - 1)) : WTF::nullopt;
+    return m_portLength ? parseInteger<uint16_t>(StringView(m_string).substring(m_hostEnd + 1, m_portLength - 1)) : std::nullopt;
 }
 
 String URL::hostAndPort() const
@@ -133,14 +165,14 @@ String URL::protocolHostAndPort() const
     );
 }
 
-static Optional<LChar> decodeEscapeSequence(StringView input, unsigned index, unsigned length)
+static std::optional<LChar> decodeEscapeSequence(StringView input, unsigned index, unsigned length)
 {
     if (index + 3 > length || input[index] != '%')
-        return WTF::nullopt;
+        return std::nullopt;
     auto digit1 = input[index + 1];
     auto digit2 = input[index + 2];
     if (!isASCIIHexDigit(digit1) || !isASCIIHexDigit(digit2))
-        return WTF::nullopt;
+        return std::nullopt;
     return toASCIIHexValue(digit1, digit2);
 }
 
@@ -215,7 +247,7 @@ String URL::fileSystemPath() const
 
     auto result = decodeEscapeSequencesFromParsedURL(path());
 #if PLATFORM(WIN)
-    result = fileSystemRepresentation(result);
+    result = FileSystem::fileSystemRepresentation(result);
 #endif
     return result;
 }
@@ -248,13 +280,13 @@ static void assertProtocolIsGood(StringView protocol)
 static Lock defaultPortForProtocolMapForTestingLock;
 
 using DefaultPortForProtocolMapForTesting = HashMap<String, uint16_t>;
-static DefaultPortForProtocolMapForTesting*& defaultPortForProtocolMapForTesting()
+static DefaultPortForProtocolMapForTesting*& defaultPortForProtocolMapForTesting() WTF_REQUIRES_LOCK(defaultPortForProtocolMapForTestingLock)
 {
     static DefaultPortForProtocolMapForTesting* defaultPortForProtocolMap;
     return defaultPortForProtocolMap;
 }
 
-static DefaultPortForProtocolMapForTesting& ensureDefaultPortForProtocolMapForTesting()
+static DefaultPortForProtocolMapForTesting& ensureDefaultPortForProtocolMapForTesting() WTF_REQUIRES_LOCK(defaultPortForProtocolMapForTestingLock)
 {
     DefaultPortForProtocolMapForTesting*& defaultPortForProtocolMap = defaultPortForProtocolMapForTesting();
     if (!defaultPortForProtocolMap)
@@ -264,25 +296,26 @@ static DefaultPortForProtocolMapForTesting& ensureDefaultPortForProtocolMapForTe
 
 void registerDefaultPortForProtocolForTesting(uint16_t port, const String& protocol)
 {
-    auto locker = holdLock(defaultPortForProtocolMapForTestingLock);
+    Locker locker { defaultPortForProtocolMapForTestingLock };
     ensureDefaultPortForProtocolMapForTesting().add(protocol, port);
 }
 
 void clearDefaultPortForProtocolMapForTesting()
 {
-    auto locker = holdLock(defaultPortForProtocolMapForTestingLock);
+    Locker locker { defaultPortForProtocolMapForTestingLock };
     if (auto* map = defaultPortForProtocolMapForTesting())
         map->clear();
 }
 
-Optional<uint16_t> defaultPortForProtocol(StringView protocol)
+std::optional<uint16_t> defaultPortForProtocol(StringView protocol)
 {
-    if (auto* overrideMap = defaultPortForProtocolMapForTesting()) {
-        auto locker = holdLock(defaultPortForProtocolMapForTestingLock);
-        ASSERT(overrideMap); // No need to null check again here since overrideMap cannot become null after being non-null.
-        auto iterator = overrideMap->find(protocol.toStringWithoutCopying());
-        if (iterator != overrideMap->end())
-            return iterator->value;
+    {
+        Locker locker { defaultPortForProtocolMapForTestingLock };
+        if (auto* overrideMap = defaultPortForProtocolMapForTesting()) {
+            auto iterator = overrideMap->find(protocol.toStringWithoutCopying());
+            if (iterator != overrideMap->end())
+                return iterator->value;
+        }
     }
     return URLParser::defaultPortForProtocol(protocol);
 }
@@ -297,6 +330,11 @@ bool URL::protocolIsJavaScript() const
     return WTF::protocolIsJavaScript(string());
 }
 
+bool URL::protocolIsInFTPFamily() const
+{
+    return WTF::protocolIsInFTPFamily(string());
+}
+
 bool URL::protocolIs(const char* protocol) const
 {
     assertProtocolIsGood(protocol);
@@ -354,7 +392,7 @@ bool URL::setProtocol(StringView newProtocol)
 {
     // Firefox and IE remove everything after the first ':'.
     auto newProtocolPrefix = newProtocol.substring(0, newProtocol.find(':'));
-    auto newProtocolCanonicalized = URLParser::maybeCanonicalizeScheme(newProtocolPrefix.toStringWithoutCopying());
+    auto newProtocolCanonicalized = URLParser::maybeCanonicalizeScheme(newProtocolPrefix);
     if (!newProtocolCanonicalized)
         return false;
 
@@ -378,22 +416,20 @@ bool URL::setProtocol(StringView newProtocol)
 // Return value of false means error in encoding.
 static bool appendEncodedHostname(Vector<UChar, 512>& buffer, StringView string)
 {
-    // Needs to be big enough to hold an IDN-encoded name.
+    // hostnameBuffer needs to be big enough to hold an IDN-encoded name.
     // For host names bigger than this, we won't do IDN encoding, which is almost certainly OK.
-    const unsigned hostnameBufferLength = 2048;
-
-    if (string.length() > hostnameBufferLength || string.isAllASCII()) {
+    if (string.length() > URLParser::hostnameBufferLength || string.isAllASCII()) {
         append(buffer, string);
         return true;
     }
 
-    UChar hostnameBuffer[hostnameBufferLength];
+    UChar hostnameBuffer[URLParser::hostnameBufferLength];
     UErrorCode error = U_ZERO_ERROR;
     UIDNAInfo processingDetails = UIDNA_INFO_INITIALIZER;
     int32_t numCharactersConverted = uidna_nameToASCII(&URLParser::internationalDomainNameTranscoder(),
-        string.upconvertedCharacters(), string.length(), hostnameBuffer, hostnameBufferLength, &processingDetails, &error);
+        string.upconvertedCharacters(), string.length(), hostnameBuffer, URLParser::hostnameBufferLength, &processingDetails, &error);
 
-    if (U_SUCCESS(error) && !processingDetails.errors) {
+    if (U_SUCCESS(error) && !(processingDetails.errors & ~URLParser::allowedNameToASCIIErrors) && numCharactersConverted) {
         buffer.append(hostnameBuffer, numCharactersConverted);
         return true;
     }
@@ -414,6 +450,18 @@ unsigned URL::credentialsEnd() const
     return end;
 }
 
+static bool forwardSlashHashOrQuestionMark(UChar c)
+{
+    return c == '/'
+        || c == '#'
+        || c == '?';
+}
+
+static bool slashHashOrQuestionMark(UChar c)
+{
+    return forwardSlashHashOrQuestionMark(c) || c == '\\';
+}
+
 void URL::setHost(StringView newHost)
 {
     if (!m_isValid)
@@ -422,20 +470,23 @@ void URL::setHost(StringView newHost)
     if (newHost.contains(':') && !newHost.startsWith('['))
         return;
 
+    if (auto index = newHost.find(hasSpecialScheme() ? slashHashOrQuestionMark : forwardSlashHashOrQuestionMark); index != notFound)
+        newHost = newHost.substring(0, index);
+
     Vector<UChar, 512> encodedHostName;
-    if (!appendEncodedHostname(encodedHostName, newHost))
+    if (hasSpecialScheme() && !appendEncodedHostname(encodedHostName, newHost))
         return;
 
     bool slashSlashNeeded = m_userStart == m_schemeEnd + 1U;
     parse(makeString(
         StringView(m_string).left(hostStart()),
         slashSlashNeeded ? "//" : "",
-        StringView(encodedHostName.data(), encodedHostName.size()),
+        hasSpecialScheme() ? StringView(encodedHostName.data(), encodedHostName.size()) : newHost,
         StringView(m_string).substring(m_hostEnd)
     ));
 }
 
-void URL::setPort(Optional<uint16_t> port)
+void URL::setPort(std::optional<uint16_t> port)
 {
     if (!m_isValid)
         return;
@@ -467,7 +518,7 @@ void URL::setHostAndPort(StringView hostAndPort)
         // Multiple colons are acceptable only in case of IPv6.
         if (hostName.contains(':') && !hostName.startsWith('['))
             return;
-        if (!parseUInt16(portString))
+        if (!parseInteger<uint16_t>(portString))
             portString = { };
     }
     if (hostName.isEmpty()) {
@@ -476,23 +527,24 @@ void URL::setHostAndPort(StringView hostAndPort)
     }
 
     Vector<UChar, 512> encodedHostName;
-    if (!appendEncodedHostname(encodedHostName, hostName))
+    if (hasSpecialScheme() && !appendEncodedHostname(encodedHostName, hostName))
         return;
 
     bool slashSlashNeeded = m_userStart == m_schemeEnd + 1U;
     parse(makeString(
         StringView(m_string).left(hostStart()),
         slashSlashNeeded ? "//" : "",
-        StringView(encodedHostName.data(), encodedHostName.size()),
+        hasSpecialScheme() ? StringView(encodedHostName.data(), encodedHostName.size()) : hostName,
         portString.isEmpty() ? "" : ":",
         portString,
         StringView(m_string).substring(pathStart())
     ));
 }
 
-static String percentEncodeCharacters(const String& input, bool(*shouldEncode)(UChar))
+template<typename StringType>
+static String percentEncodeCharacters(const StringType& input, bool(*shouldEncode)(UChar))
 {
-    auto encode = [shouldEncode] (const String& input) {
+    auto encode = [shouldEncode] (const StringType& input) {
         CString utf8 = input.utf8();
         auto* data = utf8.data();
         StringBuilder builder;
@@ -513,7 +565,10 @@ static String percentEncodeCharacters(const String& input, bool(*shouldEncode)(U
         if (UNLIKELY(shouldEncode(input[i])))
             return encode(input);
     }
-    return input;
+    if constexpr (std::is_same_v<StringType, StringView>)
+        return input.toString();
+    else
+        return input;
 }
 
 void URL::parse(const String& string)
@@ -545,7 +600,7 @@ void URL::setUser(StringView newUser)
         parse(makeString(
             StringView(m_string).left(m_userStart),
             slashSlashNeeded ? "//" : "",
-            percentEncodeCharacters(newUser.toStringWithoutCopying(), URLParser::isInUserInfoEncodeSet),
+            percentEncodeCharacters(newUser, URLParser::isInUserInfoEncodeSet),
             needSeparator ? "@" : "",
             StringView(m_string).substring(end)
         ));
@@ -567,7 +622,7 @@ void URL::setPassword(StringView newPassword)
         parse(makeString(
             StringView(m_string).left(m_userEnd),
             needLeadingSlashes ? "//:" : ":",
-            percentEncodeCharacters(newPassword.toStringWithoutCopying(), URLParser::isInUserInfoEncodeSet),
+            percentEncodeCharacters(newPassword, URLParser::isInUserInfoEncodeSet),
             '@',
             StringView(m_string).substring(credentialsEnd())
         ));
@@ -590,7 +645,7 @@ void URL::setFragmentIdentifier(StringView identifier)
     if (!m_isValid)
         return;
 
-    parse(makeString(StringView(m_string).left(m_queryEnd), '#', identifier));
+    *this = URLParser(makeString(StringView(m_string).left(m_queryEnd), '#', identifier), { }, URLTextEncodingSentinelAllowingC0AtEndOfHash).result();
 }
 
 void URL::removeFragmentIdentifier()
@@ -628,10 +683,10 @@ void URL::setQuery(StringView newQuery)
 
 static String escapePathWithoutCopying(StringView path)
 {
-    auto questionMarkOrNumberSign = [] (UChar character) {
-        return character == '?' || character == '#';
+    auto questionMarkOrNumberSignOrNonASCII = [] (UChar character) {
+        return character == '?' || character == '#' || !isASCII(character);
     };
-    return percentEncodeCharacters(path.toStringWithoutCopying(), questionMarkOrNumberSign);
+    return percentEncodeCharacters(path, questionMarkOrNumberSignOrNonASCII);
 }
 
 void URL::setPath(StringView path)
@@ -641,7 +696,8 @@ void URL::setPath(StringView path)
 
     parse(makeString(
         StringView(m_string).left(pathStart()),
-        path.startsWith('/') ? "" : "/",
+        path.startsWith('/') || (path.startsWith('\\') && (hasSpecialScheme() || protocolIs("file"))) || (!hasSpecialScheme() && path.isEmpty() && m_schemeEnd + 1U < pathStart()) ? "" : "/",
+        !hasSpecialScheme() && host().isEmpty() && path.startsWith("//") && path.length() > 2 ? "/." : "",
         escapePathWithoutCopying(path),
         StringView(m_string).substring(m_pathEnd)
     ));
@@ -682,13 +738,13 @@ bool protocolHostAndPortAreEqual(const URL& a, const URL& b)
 
     // Check the scheme
     for (unsigned i = 0; i < a.m_schemeEnd; ++i) {
-        if (a.string()[i] != b.string()[i])
+        if (toASCIILower(a.string()[i]) != toASCIILower(b.string()[i]))
             return false;
     }
 
     // And the host
     for (unsigned i = 0; i < hostLengthA; ++i) {
-        if (a.string()[hostStartA + i] != b.string()[hostStartB + i])
+        if (toASCIILower(a.string()[hostStartA + i]) != toASCIILower(b.string()[hostStartB + i]))
             return false;
     }
 
@@ -806,6 +862,17 @@ bool protocolIsJavaScript(StringView string)
     return protocolIsInternal(string, "javascript");
 }
 
+bool protocolIsInFTPFamily(StringView url)
+{
+    auto length = url.length();
+    // Do the comparison without making a new string object.
+    return length >= 4
+        && isASCIIAlphaCaselessEqual(url[0], 'f')
+        && isASCIIAlphaCaselessEqual(url[1], 't')
+        && isASCIIAlphaCaselessEqual(url[2], 'p')
+        && (url[3] == ':' || (isASCIIAlphaCaselessEqual(url[3], 's') && length >= 5 && url[4] == ':'));
+}
+
 bool protocolIsInHTTPFamily(StringView url)
 {
     auto length = url.length();
@@ -818,24 +885,25 @@ bool protocolIsInHTTPFamily(StringView url)
         && (url[4] == ':' || (isASCIIAlphaCaselessEqual(url[4], 's') && length >= 6 && url[5] == ':'));
 }
 
+
+static StaticStringImpl aboutBlankString { "about:blank" };
 const URL& aboutBlankURL()
 {
-    static NeverDestroyed<URL> staticBlankURL;
+    static LazyNeverDestroyed<URL> staticBlankURL;
     static std::once_flag onceFlag;
     std::call_once(onceFlag, [&] {
-        static constexpr const char* aboutBlank = "about:blank";
-        staticBlankURL.get() = URL(URL(), StringImpl::createStaticStringImpl(aboutBlank, strlen(aboutBlank)));
+        staticBlankURL.construct(URL(), &aboutBlankString);
     });
     return staticBlankURL;
 }
 
+static StaticStringImpl aboutSrcDocString { "about:srcdoc" };
 const URL& aboutSrcDocURL()
 {
-    static NeverDestroyed<URL> staticSrcDocURL;
+    static LazyNeverDestroyed<URL> staticSrcDocURL;
     static std::once_flag onceFlag;
     std::call_once(onceFlag, [&] {
-        static constexpr const char* aboutSrcDoc = "about:srcdoc";
-        staticSrcDocURL.get() = URL(URL(), StringImpl::createStaticStringImpl(aboutSrcDoc, strlen(aboutSrcDoc)));
+        staticSrcDocURL.construct(URL(), &aboutSrcDocString);
     });
     return staticSrcDocURL;
 }
@@ -847,7 +915,7 @@ bool URL::protocolIsAbout() const
 
 bool portAllowed(const URL& url)
 {
-    Optional<uint16_t> port = url.port();
+    std::optional<uint16_t> port = url.port();
 
     // Since most URLs don't have a port, return early for the "no port" case.
     if (!port)
@@ -915,6 +983,8 @@ bool portAllowed(const URL& url)
         587,  // ESMTP
         601,  // syslog-conn
         636,  // LDAP+SSL
+        989,  // ftps-data
+        990,  // ftps
         993,  // IMAP+SSL
         995,  // POP3+SSL
         1719, // H323 (RAS)
@@ -935,6 +1005,7 @@ bool portAllowed(const URL& url)
         6669, // Alternate IRC [Apple addition]
         6679, // Alternate IRC SSL [Apple addition]
         6697, // IRC+SSL [Apple addition]
+        10080, // amanda
     };
 
     // If the port is not in the blocked port list, allow it.
@@ -983,7 +1054,7 @@ String URL::stringCenterEllipsizedToLength(unsigned length) const
 
 URL URL::fakeURLWithRelativePart(StringView relativePart)
 {
-    return URL(URL(), makeString("webkit-fake-url://", createCanonicalUUIDString(), '/', relativePart));
+    return URL(URL(), makeString("webkit-fake-url://", createVersion4UUIDString(), '/', relativePart));
 }
 
 URL URL::fileURLWithFileSystemPath(StringView path)
@@ -1120,4 +1191,91 @@ bool URL::hostIsIPAddress(StringView host)
 
 #endif
 
+Vector<KeyValuePair<String, String>> queryParameters(const URL& url)
+{
+    return URLParser::parseURLEncodedForm(url.query());
+}
+
+Vector<KeyValuePair<String, String>> differingQueryParameters(const URL& firstURL, const URL& secondURL)
+{
+    auto firstQueryParameters = URLParser::parseURLEncodedForm(firstURL.query());
+    auto secondQueryParameters = URLParser::parseURLEncodedForm(secondURL.query());
+    
+    if (firstQueryParameters.isEmpty())
+        return secondQueryParameters;
+
+    if (secondQueryParameters.isEmpty())
+        return firstQueryParameters;
+    
+    auto compare = [] (const KeyValuePair<String, String>& a, const KeyValuePair<String, String>& b) {
+        if (int result = codePointCompare(a.key, b.key))
+            return result;
+        return codePointCompare(a.value, b.value);
+        
+    };
+    auto comparesLessThan = [compare] (const KeyValuePair<String, String>& a, const KeyValuePair<String, String>& b) {
+        return compare(a, b) < 0;
+    };
+    
+    std::sort(firstQueryParameters.begin(), firstQueryParameters.end(), comparesLessThan);
+    std::sort(secondQueryParameters.begin(), secondQueryParameters.end(), comparesLessThan);
+    size_t totalFirstQueryParameters = firstQueryParameters.size();
+    size_t totalSecondQueryParameters = secondQueryParameters.size();
+    size_t indexInFirstQueryParameters = 0;
+    size_t indexInSecondQueryParameters = 0;
+    Vector<KeyValuePair<String, String>> differingQueryParameters;
+    while (indexInFirstQueryParameters < totalFirstQueryParameters && indexInSecondQueryParameters < totalSecondQueryParameters) {
+        int comparison = compare(firstQueryParameters[indexInFirstQueryParameters], secondQueryParameters[indexInSecondQueryParameters]);
+        if (comparison < 0) {
+            differingQueryParameters.append(firstQueryParameters[indexInFirstQueryParameters]);
+            indexInFirstQueryParameters++;
+        } else if (comparison > 0) {
+            differingQueryParameters.append(secondQueryParameters[indexInSecondQueryParameters]);
+            indexInSecondQueryParameters++;
+        } else {
+            indexInFirstQueryParameters++;
+            indexInSecondQueryParameters++;
+        }
+    }
+    
+    while (indexInFirstQueryParameters < totalFirstQueryParameters) {
+        differingQueryParameters.append(firstQueryParameters[indexInFirstQueryParameters]);
+        indexInFirstQueryParameters++;
+    }
+    
+    while (indexInSecondQueryParameters < totalSecondQueryParameters) {
+        differingQueryParameters.append(secondQueryParameters[indexInSecondQueryParameters]);
+        indexInSecondQueryParameters++;
+    }
+    
+    return differingQueryParameters;
+}
+
+static StringView substringIgnoringQueryAndFragments(const URL& url)
+{
+    if (!url.isValid())
+        return StringView(url.string());
+    
+    return StringView(url.string()).left(url.pathEnd());
+}
+
+bool isEqualIgnoringQueryAndFragments(const URL& a, const URL& b)
+{
+    return substringIgnoringQueryAndFragments(a) == substringIgnoringQueryAndFragments(b);
+}
+
+void removeQueryParameters(URL& url, const HashSet<String>& keysToRemove)
+{
+    if (keysToRemove.isEmpty())
+        return;
+    
+    StringBuilder queryWithoutRemovalKeys;
+    for (auto& parameter : URLParser::parseURLEncodedForm(url.query())) {
+        if (!keysToRemove.contains(parameter.key))
+            queryWithoutRemovalKeys.append(queryWithoutRemovalKeys.isEmpty() ? "" : "&", parameter.key, '=', parameter.value);
+    }
+    
+    url.setQuery(queryWithoutRemovalKeys);
+}
+
 } // namespace WTF
